/*****************************************************************************************************/
/*                                        GetMyIP                                                    */
/*                     by Peter Lueersen <Peter@warp-ecs-owl.de>                                     */
/*                                        V 1.0.2                                                    */
/*****************************************************************************************************/
/* Application Module Name : GetMyIP								     */
/* File Name               : getmyip.cmd							     */
/* Date                    : 02.04.2005                                                              */
/* Author                  : Peter Lueersen                                                          */
/*****************************************************************************************************/
/* This program is free software                                                                     */
/* you can redistribute it and/or modify it under the terms of the GNU General Public License        */
/* as published by the Free Software Foundation; either version 2 of the License,                    */
/* or (at your option) any later version.                                                            */
/*                                                                                                   */
/* This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY          */
/* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.         */
/* See the GNU General Public License for more details.                                              */
/*                                                                                                   */
/* You should have received a copy of the GNU General Public License along with this program         */
/* if not, write to the                                                                              */
/* Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.           */
/*****************************************************************************************************/
/*                                                                                                   */
/* Description:                                                                                      */
/* This REXX script will retrieve an IP address assigned to a host either                            */
/* through an external server. The address is an external IP address which                           */
/* may be dynamically assigned by an ISP.                                                            */
/* By default the script will query a user defined IP echo server and                                */ 
/* will show the returned IP address.                                                                */
/*****************************************************************************************************/
/*-                                                                                                 -*/
/*- DEFAULT ECHO SERVER: ipid.shat.net 								    -*/
/*-												    -*/
/*- This code is based on getip.cmd by Dariusz Piatkowski 					    -*/
/*-												    -*/
/*- Modification History									    -*/
/*-												    -*/
/*-   VERSION        DATE         AUTHOR							    -*/
/*-												    -*/
/*-   V1.0.2  08.04.2006          Peter Lersen                                                     -*/
/*-     New Commanline                                                                              -*/
/*-     New Bugreport                                                                               -*/
/*-     New Help /?  	                                                                            -*/
/*-   V1.0.1        02.04.2006    Peter Lueersen						    -*/
/*-   	Fix error in line 32 :-( 								    -*/
/*-   V1.0.0        01.04.2006    Peter Lueersen						    -*/
/*-   	First version of the code.								    -*/
/*-												    -*/
/*---------------------------------------------------------------------------------------------------*/

signal on syntax name error

/* initialize socket package */
IF RxFuncQuery('SysLoadFuncs') <> 0 THEN DO
   CALL RxFuncAdd 'SysLoadFuncs','RexxUtil','SysLoadFuncs'
   loaded = 1
END
CALL SysLoadFuncs
                      

/* define some globals that we are going to use */

Globalsver='V 1.0.2'

crlf = "0d0a"x
echo_server = "ipid.shat.net"
echo_dir = "/iponly/"
find_after = "body"

/* DEBUG control flag */
debug_on = 0

/* parse the string for the ARGs */ 
PARSE ARG ARG1 ARG2 ARG3 ARG4

IF (ARG1='/?') | (ARG2='/?') | (ARG3='/?') | (ARG4='/?') THEN 
   DO
   call SysCls
   SAY "GetMyIP     Ver.: " || Globalsver || " by Peter Lueersen 2006"  
   SAY "            Peter@WARP-ECS-OWL.DE"
   SAY " "
   SAY "    Syntax         GetMyIP [/E:SERVER [/S:SIDE] /A:STRING] [/D:] [/?]"
   SAY " "
   SAY "    /?             --> This Help"
   SAY "    /E:SERVER      --> Echo-SERVER to use"
   SAY "    /S:SIDE        --> SIDE on Echoserver"
   SAY "    /A:STRING      --> Find IP after this STRING in the next 160 bytes"
   SAY "                       do not use '<' or '>' in STRING"
   SAY "    /D:            --> Debug on"
   SAY " "
   SAY "    GetMyIP        --> will use /E:" || echo_server || " /S:" || echo_dir || " /A:" || find_after 
   EXIT
   END

IF (ARG1='/D:') | (ARG2='/D:') | (ARG3='/D:') | (ARG4='/D:') THEN 
   DO
   debug_on = 1
   END

IF (substr(ARG1,1,3)='/E:') THEN 
   DO
   echo_server = substr(ARG1,4)
   echo_dir = ""
   find_after = ""
   END
IF (substr(ARG2,1,3)='/E:') THEN 
   DO
   echo_server = substr(ARG2,4)
   echo_dir = ""
   find_after = ""
   END
IF (substr(ARG3,1,3)='/E:') THEN 
   DO
   echo_server = substr(ARG3,4)
   echo_dir = ""
   find_after = ""
   END
IF (substr(ARG4,1,3)='/E:') THEN 
   DO
   echo_server = substr(ARG4,4)
   echo_dir = ""
   find_after = ""
   END

IF (substr(ARG1,1,3)='/S:') THEN 
   DO
   echo_dir = substr(ARG1,4)
   END
IF (substr(ARG2,1,3)='/S:') THEN 
   DO
   echo_dir = substr(ARG2,4)
   END
IF (substr(ARG3,1,3)='/S:') THEN 
   DO
   echo_dir = substr(ARG3,4)
   END
IF (substr(ARG4,1,3)='/S:') THEN 
   DO
   echo_dir = substr(ARG4,4)
   END

IF (substr(ARG1,1,3)='/A:') THEN 
   DO
   find_after = substr(ARG1,4)
   END
IF (substr(ARG2,1,3)='/A:') THEN 
   DO
   find_after = substr(ARG2,4)
   END
IF (substr(ARG3,1,3)='/A:') THEN 
   DO
   find_after = substr(ARG3,4)
   END
IF (substr(ARG4,1,3)='/A:') THEN 
   DO
   find_after = substr(ARG4,4)
   END

/* choose port number - http port is normally 80 */
port = 80
 
server = echo_server
URL = "GET http://"|| echo_server || echo_dir || " HTTP/1.0" || crlf || crlf

call debug_show "URL START ->" || URL || "<- URL END", debug_on
   
/* initialize socket package */
IF RxFuncQuery("SockLoadFuncs") THEN
   DO
   rc = RxFuncAdd("SockLoadFuncs","RxSock","SockLoadFuncs")
   rc = SockLoadFuncs()
   END
   
/* get server name */
rc = SockGetHostByName(server,"host.!")
IF (rc = 0) THEN
   DO
   SAY "Error" h_errno "calling SockGetHostByName("server")"
   EXIT
   END

server = host.!addr;

/* open socket */
socket  = SockSocket("AF_INET","SOCK_STREAM",0)
IF (socket = -1) THEN
   DO
   SAY "Error on SockSocket:" errno
   EXIT
   END

/* catch breaks */
SIGNAL on halt

/* connect socket */
server.!family = "AF_INET"
server.!port   = port
server.!addr   = server

rc = SockConnect(socket,"server.!")
IF (rc = -1) THEN
   DO
   SAY "Error on SockConnect:" errno
   EXIT
   END

rc = SockSend(socket, URL)
IF (rc = -1) THEN
   DO
   SAY "Error on SockSend URL:" errno
   EXIT
   END

/* receive the result from the server */
Do UNTIL rc = 0
   rc = SockRecv(socket,"newData",512)
   call debug_show "DATA START ->" || newData "<- DATA END", debug_on
   returnData = returnData || newData
END

/* now go ahead and retrieve the IP address info from the data sent back */
   location = pos(find_after, returnData) + length(find_after)
   
/* 
   return the next 160 bytes to capture the additional HTML formatting along with
   the IP address
 */
ip_address = substr(returnData, location, 160)

CALL debug_show "IP SOURCE START ->" || ip_address "<- IP SOURCE END", debug_on

/* parse the string for the IP address */ 
Parse Var ip_address ip_1 "." ip_2 "." ip_3 "." ip_4 .

/* strip off the leading junk before the first part of the IP address */
do WHILE DataType(ip_1) <> "NUM"
   ip_1 = substr(ip_1,2)
end

/* strip off the trailing junk after the last part of the IP address */
do WHILE DataType(ip_4) <> "NUM"
   ip_4 = substr(ip_4, 1, (length(ip_4)-1))
end

/* now assemble the complete IP address */
final_ip_address = ip_1 || '.' || ip_2 || '.' || ip_3 || '.' || ip_4
   
say final_ip_address

/*----------------------------------------------------------------------------*/

/* close socket (and catch signals) */

halt:

rc = SockSoClose(socket)
if (rc = -1) then
   do
   say "Error on SockSoClose:" errno
   exit
   end

exit
                   
/*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*/
/* DEBUG output wrapper function                                              */
/*----------------------------------------------------------------------------*/

debug_show: PROCEDURE
PARSE ARG message, debug_on

IF debug_on THEN 
   SAY message

RETURN

/*----------------------------------------------------------------------------*/
/* If Error :(( then show                                                     */
/*----------------------------------------------------------------------------*/
Error:
        say crlf
        say '-Hoops-'||crlf
        say 'Send this bugreport to :'
        say 'Peter@warp-ecs-owl.de      Subject: GetMyIP Error'
        say ''||crlf
        say 'Version......: ' || Globalsver
        say 'Linenumber...: ' || sigl
        say 'Linetext.....: ' || sourceline(sigl)
        say 'Errorcode....: ' || rc
        say 'Errortext....: ' || errortext(rc)
        say 'Options......: ' || ARG(1) 
        say ''||crlf
exit
